@@ -13,8 +13,6 @@
 #                   or
 #   $ ./server-version.py https://svn.collab.net/
 #
-# Python 1.5.2 or later is required.
-#
 
 import sys
 import httplib
@@ -28,7 +26,7 @@ def print_version(url):
   elif scheme == 'https':
     conn = httplib.HTTPSConnection(netloc)
   else:
-    print 'ERROR: this script only supports "http" and "https" URLs'
+    print('ERROR: this script only supports "http" and "https" URLs')
     sys.exit(1)
   conn.putrequest('OPTIONS', path)
   conn.putheader('Host', netloc)
@@ -40,24 +38,24 @@ def print_version(url):
   # Handle "OK" and Handle redirect requests, if requested resource
   # resides temporarily under a different URL
   if status != 200 and status != 302:
-    print 'ERROR: bad status response: %s %s' % (status, msg)
+    print('ERROR: bad status response: %s %s' % (status, msg))
     sys.exit(1)
   if not server:
     # a missing Server: header. Bad, bad server! Go sit in the corner!
-    print 'WARNING: missing header'
+    print('WARNING: missing header')
   else:
     for part in server.split(' '):
       if part[:4] == 'SVN/':
-        print part[4:]
+        print(part[4:])
         break
     else:
       # the server might be configured to hide this information, or it
       # might not have mod_dav_svn loaded into it.
-      print 'NOTICE: version unknown'
+      print('NOTICE: version unknown')
 
 
 if __name__ == '__main__':
   if len(sys.argv) != 2:
-    print 'USAGE: %s URL' % sys.argv[0]
+    print('USAGE: %s URL' % sys.argv[0])
     sys.exit(1)
   print_version(sys.argv[1])
