@@ -1,9 +1,10 @@
 #!/usr/local/bin/python2.5
+# -*- coding: utf-8 -*-
 
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.5.x/tools/hook-scripts/svnperms.py $
-# $LastChangedDate: 2007-08-26 01:04:48 +0000 (Sun, 26 Aug 2007) $
-# $LastChangedBy: hwright $
-# $LastChangedRevision: 26317 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.6.x/tools/hook-scripts/svnperms.py $
+# $LastChangedDate: 2008-12-24 20:49:19 +0000 (Wed, 24 Dec 2008) $
+# $LastChangedBy: arfrever $
+# $LastChangedRevision: 34929 $
 
 import commands
 import sys, os
@@ -63,8 +64,8 @@ class Config:
                     self._sections_list.append((sectname, cursectlist))
                     optname = None
                 elif cursectdict is None:
-                    raise Error, "%s:%d: no section header" % \
-                                 (filename, lineno)
+                    raise Error("%s:%d: no section header" % \
+                                 (filename, lineno))
                 else:
                     m = OPTION.match(line)
                     if m:
@@ -73,14 +74,14 @@ class Config:
                         cursectdict[optname] = optval
                         cursectlist.append([optname, optval])
                     else:
-                        raise Error, "%s:%d: parsing error" % \
-                                     (filename, lineno)
+                        raise Error("%s:%d: parsing error" % \
+                                     (filename, lineno))
 
     def sections(self):
-        return self._sections_dict.keys()
+        return list(self._sections_dict.keys())
 
     def options(self, section):
-        return self._sections_dict.get(section, {}).keys()
+        return list(self._sections_dict.get(section, {}).keys())
 
     def get(self, section, option, default=None):
         return self._sections_dict.get(option, default)
@@ -120,8 +121,8 @@ class Permission:
                         try:
                             users.extend(self._group[groupuser[1:]])
                         except KeyError:
-                            raise Error, "group '%s' not found" % \
-                                         groupuser[1:]
+                            raise Error("group '%s' not found" % \
+                                         groupuser[1:])
                     else:
                         users.append(groupuser)
                 self._permlist.append((pattern, users, perms))
@@ -146,7 +147,7 @@ class SVNLook:
             sys.stderr.write(cmdstr)
             sys.stderr.write("\n")
             sys.stderr.write(output)
-            raise Error, "command failed: %s\n%s" % (cmdstr, output)
+            raise Error("command failed: %s\n%s" % (cmdstr, output))
         return status, output
 
     def _execsvnlook(self, cmd, *args, **kwargs):
@@ -156,18 +157,18 @@ class SVNLook:
         execcmd_kwargs = {}
         keywords = ["show", "noerror"]
         for key in keywords:
-            if kwargs.has_key(key):
+            if key in kwargs:
                 execcmd_kwargs[key] = kwargs[key]
         return self._execcmd(*execcmd_args, **execcmd_kwargs)
 
     def _add_txnrev(self, cmd_args, received_kwargs):
-        if received_kwargs.has_key("txn"):
+        if "txn" in received_kwargs:
             txn = received_kwargs.get("txn")
             if txn is not None:
                 cmd_args += ["-t", txn]
         elif self.txn is not None:
             cmd_args += ["-t", self.txn]
-        if received_kwargs.has_key("rev"):
+        if "rev" in received_kwargs:
             rev = received_kwargs.get("rev")
             if rev is not None:
                 cmd_args += ["-r", rev]
@@ -207,9 +208,9 @@ def check_perms(filename, section, repos, txn=None, rev=None, author=None):
     try:
         config = Config(filename)
     except IOError:
-        raise Error, "can't read config file "+filename
+        raise Error("can't read config file "+filename)
     if not section in config.sections():
-        raise Error, "section '%s' not found in config file" % section
+        raise Error("section '%s' not found in config file" % section)
     perm = Permission()
     perm.parse_groups(config.walk("groups"))
     perm.parse_groups(config.walk(section+" groups"))
@@ -231,7 +232,7 @@ def check_perms(filename, section, repos, txn=None, rev=None, author=None):
     if permerrors:
         permerrors.insert(0, "you don't have enough permissions for "
                              "this transaction:")
-        raise Error, "\n".join(permerrors)
+        raise Error("\n".join(permerrors))
 
 
 # Command:
@@ -259,7 +260,7 @@ def parse_options():
     try:
         opts, args = my_getopt(sys.argv[1:], "f:s:r:t:R:A:h", ["help"])
     except getopt.GetoptError, e:
-        raise Error, e.msg
+        raise Error(e.msg)
     class Options: pass
     obj = Options()
     obj.filename = None
@@ -290,8 +291,7 @@ def parse_options():
     if not (obj.transaction or obj.revision):
         missingopts.append("either transaction or a revision")
     if missingopts:
-        raise MissingArgumentsException, \
-              "missing required option(s): " + ", ".join(missingopts)
+        raise MissingArgumentsException("missing required option(s): " + ", ".join(missingopts))
     obj.repository = os.path.abspath(obj.repository)
     if obj.filename is None:
         obj.filename = os.path.join(obj.repository, "conf", "svnperms.conf")
@@ -301,8 +301,8 @@ def parse_options():
             os.path.isdir(os.path.join(obj.repository, "db")) and
             os.path.isdir(os.path.join(obj.repository, "hooks")) and
             os.path.isfile(os.path.join(obj.repository, "format"))):
-        raise Error, "path '%s' doesn't look like a repository" % \
-                     obj.repository
+        raise Error("path '%s' doesn't look like a repository" % \
+                     obj.repository)
 
     return obj
 
