@@ -3,7 +3,7 @@
 # svn-backup-dumps.py -- Create dumpfiles to backup a subversion repository.
 #
 # ====================================================================
-# Copyright (c) 2006 CollabNet.  All rights reserved.
+# Copyright (c) 2006-2009 CollabNet.  All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.  The terms
@@ -19,8 +19,6 @@
 # This script creates dump files from a subversion repository.
 # It is intended for use in cron jobs and post-commit hooks.
 #
-# Tested on UNIX with python 2.3 and 2.4, on Windows with python 2.4.
-#
 # The basic operation modes are:
 #    1. Create a full dump (revisions 0 to HEAD).
 #    2. Create incremental dumps containing at most N revisions.
@@ -141,6 +139,7 @@ import gzip
 import os.path
 from optparse import OptionParser
 from ftplib import FTP
+from subprocess import Popen, PIPE
 
 try:
     import bz2
@@ -149,133 +148,6 @@ except ImportError:
     have_bz2 = False
 
 
-class Popen24Compat:
-
-    def __init__(self, args, bufsize=0, executable=None, stdin=None,
-            stdout=None, stderr=None, preexec_fn=None, close_fds=False,
-            shell=False, cwd=None, env=None, universal_newlines=False,
-            startupinfo=None, creationflags=0):
-
-        if isinstance(args, list):
-            args = tuple(args)
-        elif not isinstance(args, tuple):
-            raise RipperException, "Popen24Compat: args is not tuple or list"
-
-        self.stdin = None
-        self.stdout = None
-        self.stderr = None
-        self.returncode = None
-
-        if executable == None:
-            executable = args[0]
-
-        if stdin == PIPE:
-            stdin, stdin_fd = os.pipe()
-            self.stdin = os.fdopen(stdin_fd)
-        elif stdin == None:
-            stdin = 0
-        else:
-            stdin = stdin.fileno()
-        if stdout == PIPE:
-            stdout_fd, stdout = os.pipe()
-            self.stdout = os.fdopen(stdout_fd)
-        elif stdout == None:
-            stdout = 1
-        else:
-            stdout = stdout.fileno()
-        if stderr == PIPE:
-            stderr_fd, stderr = os.pipe()
-            self.stderr = os.fdopen(stderr_fd)
-        elif stderr == None:
-            stderr = 2
-        else:
-            stderr = stderr.fileno()
-
-        # error pipe
-        err_read, err_write = os.pipe()
-        fcntl.fcntl(err_write, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-
-        self.pid = os.fork()
-        if self.pid < 0:
-            raise Exception, "Popen24Compat: fork"
-        if self.pid == 0:
-            # child
-            os.close(err_read)
-            fcntl.fcntl(err_write, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-            if self.stdin:
-                self.stdin.close()
-            if self.stdout:
-                self.stdout.close()
-            if self.stderr:
-                self.stderr.close()
-            if stdin != 0:
-                os.dup2(stdin, 0)
-                os.close(stdin)
-            if stdout != 1:
-                os.dup2(stdout, 1)
-                os.close(stdout)
-            if stderr != 2:
-                os.dup2(stderr, 2)
-                os.close(stderr)
-            try:
-                if shell:
-                    # should spawn a shell here...
-                    os.execvp(executable, args)
-                else:
-                    os.execvp(executable, args)
-            except:
-                err = sys.exc_info()[1]
-            # exec error
-            os.write(err_write, str(err))
-            os._exit(255)
-        else:
-            # parent
-            os.close(err_write)
-            if stdin != 0:
-                os.close(stdin)
-            if stdout != 0:
-                os.close(stdout)
-            if stderr != 0:
-                os.close(stderr)
-            sr, sw, se = select.select([ err_read ], [], [ err_read ])
-            if len(se) == 1:
-                os.close(err_read)
-                raise Exception, "Popen24Compat: err pipe read error"
-            if len(sr) == 1:
-                err = os.read(err_read, 1024)
-            os.close(err_read)
-            if len(err) != 0:
-                raise Exception, "Popen24Compat: exec error: " + err
-
-    def poll(self):
-        self.__wait(os.WNOHANG)
-        return self.returncode
-
-    def wait(self):
-        self.__wait(0)
-        return self.returncode
-
-    def __wait(self, options):
-        pid, rc = os.waitpid(self.pid, options)
-        if pid != 0:
-            self.returncode = rc
-
-def PopenConstr(args, bufsize=0, executable=None, stdin=None, stdout=None,
-            stderr=None, preexec_fn=None, close_fds=False, shell=False,
-            cwd=None, env=None, universal_newlines=False, startupinfo=None,
-            creationflags=0):
-    return Popen24Compat(args, bufsize=bufsize, executable=executable,
-                stdin=stdin, stdout=stdout, stderr=stderr,
-                preexec_fn=preexec_fn, close_fds=close_fds, shell=shell,
-                cwd=cwd, env=env, universal_newlines=universal_newlines,
-                startupinfo=startupinfo, creationflags=creationflags)
-
-try:
-    from subprocess import Popen, PIPE
-except ImportError:
-    Popen = PopenConstr
-    PIPE = -1
-
 class SvnBackupOutput:
 
     def __init__(self, abspath, filename):
@@ -361,11 +233,9 @@ class SvnBackup:
         # need 3 args: progname, reposname, dumpdir
         if len(args) != 3:
             if len(args) < 3:
-                raise SvnBackupException, \
-                    "too few arguments, specify repospath and dumpdir."
+                raise SvnBackupException("too few arguments, specify repospath and dumpdir.")
             else:
-                raise SvnBackupException, \
-                    "too many arguments, specify repospath and dumpdir only."
+                raise SvnBackupException("too many arguments, specify repospath and dumpdir only.")
         self.__repospath = args[1]
         self.__dumpdir = args[2]
         # check repospath
@@ -374,28 +244,22 @@ class SvnBackup:
             # repospath without trailing slash
             self.__repospath = rpathparts[0]
         if not os.path.exists(self.__repospath):
-            raise SvnBackupException, \
-                "repos '%s' does not exist." % self.__repospath
+            raise SvnBackupException("repos '%s' does not exist." % self.__repospath)
         if not os.path.isdir(self.__repospath):
-            raise SvnBackupException, \
-                "repos '%s' is not a directory." % self.__repospath
+            raise SvnBackupException("repos '%s' is not a directory." % self.__repospath)
         for subdir in [ "db", "conf", "hooks" ]:
             dir = os.path.join(self.__repospath, "db")
             if not os.path.isdir(dir):
-                raise SvnBackupException, \
-                    "repos '%s' is not a repository." % self.__repospath
+                raise SvnBackupException("repos '%s' is not a repository." % self.__repospath)
         rpathparts = os.path.split(self.__repospath)
         self.__reposname = rpathparts[1]
         if self.__reposname in [ "", ".", ".." ]:
-            raise SvnBackupException, \
-                "couldn't extract repos name from '%s'." % self.__repospath
+            raise SvnBackupException("couldn't extract repos name from '%s'." % self.__repospath)
         # check dumpdir
         if not os.path.exists(self.__dumpdir):
-            raise SvnBackupException, \
-                "dumpdir '%s' does not exist." % self.__dumpdir
+            raise SvnBackupException("dumpdir '%s' does not exist." % self.__dumpdir)
         elif not os.path.isdir(self.__dumpdir):
-            raise SvnBackupException, \
-                "dumpdir '%s' is not a directory." % self.__dumpdir
+            raise SvnBackupException("dumpdir '%s' is not a directory." % self.__dumpdir)
         # set options
         self.__rev_nr = options.rev
         self.__count = options.cnt
@@ -413,14 +277,11 @@ class SvnBackup:
             self.__transfer = options.transfer.split(":")
             if len(self.__transfer) != 5:
                 if len(self.__transfer) < 5:
-                    raise SvnBackupException, \
-                        "too few fields for transfer '%s'." % self.__transfer
+                    raise SvnBackupException("too few fields for transfer '%s'." % self.__transfer)
                 else:
-                    raise SvnBackupException, \
-                        "too many fields for transfer '%s'." % self.__transfer
+                    raise SvnBackupException("too many fields for transfer '%s'." % self.__transfer)
             if self.__transfer[0] not in [ "ftp", "smb" ]:
-                raise SvnBackupException, \
-                    "unknown transfer method '%s'." % self.__transfer[0]
+                raise SvnBackupException("unknown transfer method '%s'." % self.__transfer[0])
 
     def set_nonblock(self, fileobj):
         fd = fileobj.fileno()
@@ -459,7 +320,7 @@ class SvnBackup:
                         bufout += buf
                 else:
                     if printerr:
-                        print buf,
+                        sys.stdout.write("%s " % buf)
                     else:
                         buferr += buf
             if len(readfds) == 0:
@@ -467,7 +328,7 @@ class SvnBackup:
             selres = select.select(readfds, [], [])
         rc = proc.wait()
         if printerr:
-            print ""
+            print("")
         return (rc, bufout, buferr)
 
     def exec_cmd_nt(self, cmd, output=None, printerr=False):
@@ -495,7 +356,7 @@ class SvnBackup:
         if r[0] == 0 and len(r[2]) == 0:
             return int(r[1].strip())
         else:
-            print r[2]
+            print(r[2])
         return -1
 
     def transfer_ftp(self, absfilename, filename):
@@ -513,9 +374,8 @@ class SvnBackup:
             rc = len(ifd.read(1)) == 0
             ifd.close()
         except Exception, e:
-            raise SvnBackupException, \
-                "ftp transfer failed:\n  file:  '%s'\n  error: %s" % \
-                    (absfilename, str(e))
+            raise SvnBackupException("ftp transfer failed:\n  file:  '%s'\n  error: %s" % \
+                    (absfilename, str(e)))
         return rc
 
     def transfer_smb(self, absfilename, filename):
@@ -530,7 +390,7 @@ class SvnBackup:
         r = self.exec_cmd(cmd)
         rc = r[0] == 0
         if not rc:
-            print r[2]
+            print(r[2])
         return rc
 
     def transfer(self, absfilename, filename):
@@ -541,7 +401,7 @@ class SvnBackup:
         elif self.__transfer[0] == "smb":
             self.transfer_smb(absfilename, filename)
         else:
-            print "unknown transfer method '%s'." % self.__transfer[0]
+            print("unknown transfer method '%s'." % self.__transfer[0])
 
     def create_dump(self, checkonly, overwrite, fromrev, torev=None):
         revparam = "%d" % fromrev
@@ -564,12 +424,12 @@ class SvnBackup:
             return os.path.exists(absfilename)
         elif os.path.exists(absfilename):
             if overwrite:
-                print "overwriting " + absfilename
+                print("overwriting " + absfilename)
             else:
-                print "%s already exists." % absfilename
+                print("%s already exists." % absfilename)
                 return True
         else:
-            print "writing " + absfilename
+            print("writing " + absfilename)
         cmd = [ "svnadmin", "dump",
                 "--incremental", "-r", revparam, self.__repospath ]
         if self.__quiet:
@@ -665,26 +525,26 @@ if __name__ == "__main__":
                        help="shows detailed help for the transfer option.")
     (options, args) = parser.parse_args(sys.argv)
     if options.help_transfer:
-        print "Transfer help:"
-        print ""
-        print "  FTP:"
-        print "    -t ftp:<host>:<user>:<password>:<dest-path>"
-        print ""
-        print "  SMB (using smbclient):"
-        print "    -t smb:<share>:<user>:<password>:<dest-path>"
-        print ""
+        print("Transfer help:")
+        print("")
+        print("  FTP:")
+        print("    -t ftp:<host>:<user>:<password>:<dest-path>")
+        print("")
+        print("  SMB (using smbclient):")
+        print("    -t smb:<share>:<user>:<password>:<dest-path>")
+        print("")
         sys.exit(0)
     rc = False
     try:
         backup = SvnBackup(options, args)
         rc = backup.execute()
     except SvnBackupException, e:
-        print "svn-backup-dumps.py:", e
+        print("svn-backup-dumps.py: %s" % e)
     if rc:
-        print "Everything OK."
+        print("Everything OK.")
         sys.exit(0)
     else:
-        print "An error occured!"
+        print("An error occured!")
         sys.exit(1)
 
 # vim:et:ts=4:sw=4
